local metadata = {
"## Interface: 1.4.4.0 1.4.4RC8",
"## Title: AdvancedAnimalTypes",
"## Notes: FS2017 Script for registering New Animals and their FoodGroups, via modDesc.xml",
"## Author: Blacksheep-(RC-Devil)",
"## Version: 1.0.0.0",
"## Date: 07.06.2017",
"## Web-Support: https://www.mods-community.de"
}


-- <AdvancedAnimalTypes hudsDir="animals_scripts/animalHuds/" animalStoreDir="animals_scripts/animalStore/">
--	    <addNewAnimal name="cow2" shoptitle="cow2" price="1000" mass="600" dailyUpkeep="11" canBeBought="true" hasStatistics="true" subType="0" milkPerDay="714" manurePerDay="200" liquidManurePerDay="250" palletFillLevelPerDay="0" pickUpObjectsPerDay="0" dirtFillLevelPerDay="30" birthRatePerDay="50" strawPerDay="70" waterPerDay="35" foodPerDay="350" />
--		<newAnimalfoodgroup animalname="cow2" groupname="grass" weight="0.50" filltypenames="GRASS_WINDROW" foodMixture="false" />
--		<newAnimalfoodgroup animalname="cow2" groupname="bulk" weight="0.25" filltypenames="DRYGRASS_WINDROW SILAGE" foodMixture="false" />
--		<newAnimalfoodgroup animalname="cow2" groupname="power" weight="0.25" filltypenames="POWERFOOD FORAGE" foodMixture="true" mixtureFilltype="FORAGE" mixtureGroupWeights="0, 0.5, 0.5" />
--		<newAnimalfoodgroup animalname="cow2" groupname="myfeed" weight="0.50" filltypenames="WHEAT BARLEY" foodMixture="false" />
--	</AdvancedAnimalTypes>


local DebugEbene = 0;
local function getmdata(v) v="## "..v..": "; for i=1,table.getn(metadata) do local _,n=string.find(metadata[i],v);if n then return (string.sub (metadata[i], n+1)); end;end;end;
local function Debug(e,s,...) if e <= DebugEbene then print((getmdata("Title")).." v"..(getmdata("Version"))..": "..string.format(s,...)); end; end;
local function L(name) local t = getmdata("Title"); return g_i18n:hasText(t.."_"..name) and g_i18n:getText(t.."_"..name) or name; end

AdvancedAnimalTypes = {};

AdvancedAnimalTypes.version = '1.0.0.0';
AdvancedAnimalTypes.author = 'Blacksheep(RC-Devil)';
AdvancedAnimalTypes.web = 'Web-Support: https://www.mods-community.de';
AdvancedAnimalTypes.date = '07 June 2017';
local natDir = g_currentModDirectory;

function AdvancedAnimalTypes:start(natDir)
	if self.initialized then
	    return 
	end;
	local natDir = g_currentModDirectory;
	local xmlFilePath =  Utils.getFilename('modDesc.xml', natDir);
	if fileExists(xmlFilePath) then
		local xmlFile = loadXMLFile('modDescXML', xmlFilePath);
		local key = 'modDesc.AdvancedAnimalTypes';
		if hasXMLProperty(xmlFile, key) then
			local hudsDir = getXMLString(xmlFile, key..'#hudsDir');
			local animalStoreDir = getXMLString(xmlFile, key..'#animalStoreDir');
			if hudsDir ~= nil or animalStoreDir ~= nil then
            	if hudsDir:sub(-1) ~= '/' then
            		hudsDir = hudsDir .. '/';
            	end;
            	hudsDir = natDir .. hudsDir;
            	if animalStoreDir:sub(-1) ~= '/' then
            		animalStoreDir = animalStoreDir .. '/';
				end;
            	animalStoreDir = natDir .. animalStoreDir;
				self:registerAnimal(xmlFile, key, hudsDir, animalStoreDir);
			else
				if hudsDir == nil then print('\\__ Error: AdvancedAnimalTypes could not find Huds Directory.');end;
				if animalStoreDir == nil then print('\\__ Error: AdvancedAnimalTypes could not find GroundTip Directory.');end;
			end;
			Debug(-1,('\\__ AdvancedAnimalTypes V%s by %s from %s was loaded successful, for %s'):format(AdvancedAnimalTypes.version, AdvancedAnimalTypes.author, AdvancedAnimalTypes.date, AdvancedAnimalTypes.web));
		end; 
		delete(xmlFile);
	end;
	self.initialized = true;
	
	local lcount = 0;
	local modDesc = loadXMLFile("modDesc", natDir .. "modDesc.xml");
	while true do
		local animaGuiText = string.format("modDesc.l10n.text(%d)", lcount);
		if not hasXMLProperty(modDesc, animaGuiText) then
			break;
		end;
		local name = getXMLString(modDesc, animaGuiText .. "#name");
		local entry = getXMLString(modDesc, animaGuiText .. "."..g_languageShort);
		g_i18n.globalI18N.texts[name] = Utils.getNoNil(entry,g_i18n:getText(name));
		lcount = lcount + 1;
	end;
end;

function AdvancedAnimalTypes:registerAnimal(xmlFile, key, hudsDir, animalStoreDir)
    local natDir = g_currentModDirectory;
    Debug(-1,('\\__ AdvancedAnimalTypes V%s by %s from %s is loading'):format(AdvancedAnimalTypes.version, AdvancedAnimalTypes.author, AdvancedAnimalTypes.date));

    local a = 0
	    while true do
		    local AnimalTypeKey = key .. ('.addNewAnimal(%d)'):format(a);
		        if not hasXMLProperty(xmlFile, AnimalTypeKey) then
			        break;
		        end;
		    local name = getXMLString(xmlFile, AnimalTypeKey..'#name');
		        if name == nil then
			        print(('\\__ Error: missing "name" attribute for addNewAnimal #%d in "AdvancedAnimalTypes". Adding AnimalType aborted.'):format(a));
			        break;
		        end;
		    local AnimalKey = 'ANIMAL_'..string.upper(name);
		    if AnimalUtil[AnimalKey] == nil then
			        local Animal = {
			        name = name,
			        index = AnimalUtil.NUM_ANIMALS,
			        shoptitle             = Utils.getNoNil(getXMLString(xmlFile, AnimalTypeKey..'#shoptitle'), 'No'),
			        price                 = Utils.getNoNil(getXMLFloat(xmlFile, AnimalTypeKey..'#price'), 0),
			        mass                  = Utils.getNoNil(getXMLFloat(xmlFile, AnimalTypeKey..'#mass'), 0),
			        dailyUpkeep           = Utils.getNoNil(getXMLFloat(xmlFile, AnimalTypeKey..'#dailyUpkeep'), 0),
			        canBeBought           = Utils.getNoNil(getXMLBool(xmlFile, AnimalTypeKey..'#canBeBought'), false),
			        hasStatistics         = Utils.getNoNil(getXMLBool(xmlFile, AnimalTypeKey..'#hasStatistics'), false),
			        subType               = Utils.getNoNil(getXMLFloat(xmlFile, AnimalTypeKey..'#'), 0),
			        milkPerDay            = Utils.getNoNil(getXMLFloat(xmlFile, AnimalTypeKey..'#milkPerDay'), 0),
			        manurePerDay          = Utils.getNoNil(getXMLFloat(xmlFile, AnimalTypeKey..'#manurePerDay'), 0),
			        liquidManurePerDay    = Utils.getNoNil(getXMLFloat(xmlFile, AnimalTypeKey..'#liquidManurePerDay'), 0),
			        palletFillLevelPerDay = Utils.getNoNil(getXMLFloat(xmlFile, AnimalTypeKey..'#palletFillLevelPerDay'), 0),
			        pickUpObjectsPerDay   = Utils.getNoNil(getXMLFloat(xmlFile, AnimalTypeKey..'#pickUpObjectsPerDay'), 0),
			        dirtFillLevelPerDay   = Utils.getNoNil(getXMLFloat(xmlFile, AnimalTypeKey..'#dirtFillLevelPerDay'), 0),
			        birthRatePerDay       = Utils.getNoNil(getXMLFloat(xmlFile, AnimalTypeKey..'#birthRatePerDay'), 0),
			        strawPerDay           = Utils.getNoNil(getXMLFloat(xmlFile, AnimalTypeKey..'#strawPerDay'), 0),
			        waterPerDay           = Utils.getNoNil(getXMLFloat(xmlFile, AnimalTypeKey..'#waterPerDay'), 0),
			        foodPerDay            = Utils.getNoNil(getXMLFloat(xmlFile, AnimalTypeKey..'#foodPerDay'), 0)
			        };
			local localName = Animal.name;
			    if g_i18n:hasText(Animal.name) then
				    localName = g_i18n:getText(Animal.name);
				    g_i18n.globalI18N.texts[Animal.name] = localName;
			    end;
			local animalStoreIcon = ('%sanimalStoreIcon_%s.dds'):format(animalStoreDir, Animal.name);
			    if not fileExists(Utils.getFilename(animalStoreIcon, self.amtDir)) then
				    print("\\__ "..animalStoreIcon.." These are *not* 100%, please check your AnimalStore directory or your StoreIcon file")
				    animalStoreIcon = ('%sno_animalStoreIcon.dds'):format(animalStoreDir);
				end;
			
			local hudFile = ('%shud_animal_%s.dds'):format(hudsDir, Animal.name);
			local hudFile_small = ('%shud_animal_%s_small.dds'):format(hudsDir, Animal.name);
			    if not fileExists(Utils.getFilename(hudFile_small, self.amtDir)) then
				    print("\\__ "..hudFile_small.." These are *not* 100%, please check your huds directory or your hud file")
				    hudFile_small = hudFile;
			    end;
			local animalIndex = AnimalUtil.registerAnimal(Animal.name, localName, animalStoreIcon, hudFile, hudFile_small, Animal.price, Animal.mass * 0.000001* 0.5, Animal.dailyUpkeep, Animal.canBeBought, Animal.hasStatistics)
			    AnimalUtil.setAnimalData(animalIndex, Animal.milkPerDay, Animal.manurePerDay, Animal.liquidManurePerDay, Animal.palletFillLevelPerDay, Animal.pickUpObjectsPerDay, Animal.dirtFillLevelPerDay, Animal.birthRatePerDay, Animal.strawPerDay, Animal.waterPerDay, Animal.foodPerDay)

			    Debug(1,('\\_ AdvancedAnimalType %s Debug Start'):format(Animal.name));
			    Debug(1,('\\_ Adding Animal %s (%s) Key[%s] '):format(localName, Animal.name, animalIndex));			  
			    Debug(1,('\\___ Adding StoreImage %s (%s)'):format(Animal.name, animalStoreIcon));
			    Debug(1,('\\___ Set Price for %s (%s)'):format(Animal.name, Animal.price));
			    Debug(1,('\\___ Set Mass for %s (%s)'):format(Animal.name, Animal.mass * 0.000001* 0.5));
			    Debug(1,('\\___ Set Daily Upkeep for %s (%s)'):format(Animal.name, Animal.dailyUpkeep));
			    Debug(1,('\\___ Set can Bought for %s (%s)'):format(Animal.name, Animal.canBeBought));
			    Debug(1,('\\___ Set Statistics for %s (%s)'):format(Animal.name, Animal.hasStatistics));
				
				if Animal.subType ~= 0 then
				    Debug(1,('\\___ Set Subtype for %s (%s)'):format(Animal.name, Animal.subType));
			    end;
			    if Animal.milkPerDay ~= 0 then	
			        Debug(1,('\\___ Set Milk per Day for %s (%s)'):format(Animal.name, Animal.milkPerDay));
			    end;
			    if Animal.manurePerDay ~= 0 then
			        Debug(1,('\\___ Set Manure per Day for %s (%s)'):format(Animal.name, Animal.manurePerDay));
			    end;
			    if Animal.liquidManurePerDay ~= 0 then
			        Debug(1,('\\___ Set LiquidManure per Day for %s (%s)'):format(Animal.name, Animal.liquidManurePerDay));
			    end;
			    if Animal.palletFillLevelPerDay ~= 0 then
			        Debug(1,('\\___ Set Pallet Filllevel per Day for %s (%s)'):format(Animal.name, Animal.palletFillLevelPerDay));
			    end;
			    if Animal.pickUpObjectsPerDay ~= 0 then
			        Debug(1,('\\___ Set PickUp Objects per Day for %s (%s)'):format(Animal.name, Animal.pickUpObjectsPerDay));
			    end;
			    if Animal.dirtFillLevelPerDay ~= 0 then
			        Debug(1,('\\___ Set Dirt Filllevel per Day for %s (%s)'):format(Animal.name, Animal.dirtFillLevelPerDay));
			    end;
			    if Animal.birthRatePerDay ~= 0 then
			        Debug(1,('\\___ Set Birthrate per Day for %s (%s)'):format(Animal.name, Animal.birthRatePerDay));
			    end;
			    if Animal.strawPerDay ~= 0 then
			        Debug(1,('\\___ Set Straw per Day for %s (%s)'):format(Animal.name, Animal.strawPerDay));
			    end;
			    if Animal.waterPerDay ~= 0 then
			        Debug(1,('\\___ Set Water per Day for %s (%s)'):format(Animal.name, Animal.waterPerDay));
			    end;
			    if Animal.foodPerDay ~= 0 then
			        Debug(1,('\\___ Set Food per Day for %s (%s)'):format(Animal.name, Animal.foodPerDay));
			    end;
			        Debug(1,'\\___________________________________________________________________________________________');
			    a = a + 1
		    else
			    a = a + 1
			        print(('\\__ Error: Animal %a already exists. "AdvancedAnimalTypes" will skip its registration.'):format(name));
		    end;
        end;
	 
	 
	local b = 0
	    while true do
		    local animalFoodGroupKey = key .. ('.newAnimalFoodGroup(%d)'):format(b);
		    if not hasXMLProperty(xmlFile, animalFoodGroupKey) then
			    break;
		    end;		 
		    local animalname = getXMLString(xmlFile, animalFoodGroupKey..'#animalname');
		    if animalname == nil then
			    print(('\\__ Error: missing "animalname" attribute for animalfoodgroup #%d in "AdvancedAnimalTypes". Adding FoodGroup aborted.'):format(b));
			    break;
		    end;
		    local animalSet = "ANIMAL_"..string.upper(animalname);
		  	local groupType = {
				animalname = animalname,
				groupname =  Utils.getNoNil(getXMLString(xmlFile, animalFoodGroupKey..'#groupname'), bulk),
				weight =     Utils.getNoNil(getXMLFloat(xmlFile, animalFoodGroupKey..'#weight'), 0.50),
				filltypenames =  Utils.getNoNil(getXMLString(xmlFile, animalFoodGroupKey..'#filltypenames'), wheat)
			    };
				local localName = groupType.groupname;
			    if g_i18n:hasText("fillTypeGroup_" .. groupType.groupname) then
				    localName = g_i18n:getText("fillTypeGroup_" .. groupType.groupname);
				    g_i18n.globalI18N.texts["fillTypeGroup_" .. groupType.groupname] = localName;
				end;
				local foodGroupIndex = FillUtil.registerFoodGroup(AnimalUtil[animalSet], groupType.groupname, localName, groupType.weight)
				local filltypename = Utils.splitString(" ", groupType.filltypenames);
				    for _,filltypename in pairs(filltypename) do				  
				        local foFill = 'FILLTYPE_'..string.upper(filltypename);				  
				        if AnimalUtil[animalSet] then
				            FillUtil.registerFillTypeInFoodGroup(AnimalUtil[animalSet], foodGroupIndex, FillUtil[foFill]);
				        end;
				    end;
				    Debug(2,('\\_ Register FoodGroup for %s Debug Start'):format(groupType.animalname));
				    Debug(2,('\\__ Register FoodGroup: %s for (%s) with Weight %s'):format(localName, groupType.animalname, groupType.weight));
				    Debug(2,('\\_____ Added FillType(s): %s for (%s) at Group %s'):format(groupType.filltypenames, groupType.animalname, localName));
				    Debug(2,('\\_ Register FoodGroup for %s Debug Ende'):format(groupType.animalname));
				    Debug(2,'\\___________________________________________________________________________________________'); 
			b = b + 1
        end;
		
end
AdvancedAnimalTypes:start(natDir);
